package trading;

import java.io.File;
import java.io.PrintWriter;
import java.util.ArrayList;
import java.util.Scanner;

public class Market {

    private ArrayList<Seller> sellers;
    private ArrayList<Buyer> buyers;
    private ArrayList<Sale> sales;
    
    public Market() {
        sellers = new ArrayList<Seller>();
        buyers = new ArrayList<Buyer>();
        sales = new ArrayList<Sale>();
    }
    
    public void readOrders() {
        Scanner input = null;
        try {
            input = new Scanner(new File("orders.txt"));
        } catch(Exception ex) {
            System.out.println("Could not open order file.");
            System.exit(1);
        }

        while(input.hasNext()) {
            int id = input.nextInt();
            String what = input.next();
            int quantity = input.nextInt();

            if(what.equals("buy")) {
                Buyer b = new Buyer(id,quantity);
                buyers.add(b);
            } else {
                Seller s = new Seller(id,quantity);
                sellers.add(s);
            }
        }

        input.close();
    }
    
    public void makeTrades() {
        while(sellers.size() > 0 && buyers.size() > 0) {
            // Bring together the seller at the front of the sellers line
            // with the buyer at the front of the line of buyers
            Seller s = sellers.get(0);
            Buyer b = buyers.get(0);
            if(s.getQuantity() > b.getQuantity()) {
                Sale sale = new Sale(s.getId(),b.getId(),b.getQuantity());
                sales.add(sale);
                buyers.remove(0);
                s.reduceQuantity(b.getQuantity());
            } else if(s.getQuantity() == b.getQuantity()){
                Sale sale = new Sale(s.getId(),b.getId(),b.getQuantity());
                sales.add(sale);
                sellers.remove(0);
                buyers.remove(0);
            } else {
                Sale sale = new Sale(s.getId(),b.getId(),s.getQuantity());
                sales.add(sale);
                sellers.remove(0);
                b.reduceQuantity(s.getQuantity());
            }
        }
    }
    
    public void saveSales() {
        PrintWriter pw = null;
        try {
            pw = new PrintWriter(new File("sales.txt"));
        } catch (Exception ex) {
            System.out.println("Can not open file for writing.");
            System.exit(0);
        }

        for(Sale s : sales) {
            s.writeTo(pw);
        }

        pw.close();
    }
    
    public static void main(String[] args) {
        Market m = new Market();
        m.readOrders();
        m.makeTrades();
        m.saveSales();
    }
    
}
